@@ -1,12 +1,12 @@
 package Parse::Dia::SQL::Output::Sas;
 
-# $Id: Sas.pm,v 1.1 2009/02/23 07:36:17 aff Exp $
+# $Id: Sas.pm,v 1.2 2009/02/24 09:58:24 aff Exp $
 
 =pod
 
 =head1 NAME 
 
-Informix.pm - Create SQL for Informix.
+Sas.pm - Create SQL for Sas.
 
 =head1 SEE ALSO
 
@@ -36,9 +36,9 @@ sub new {
   my ( $class, %param ) = @_;
   my $self = {};
 
-  # Set defaults for informix
-  $param{db} = q{informix}; 
-  $param{object_name_max_length} = $param{object_name_max_length} || 30;
+  # Set defaults for sas
+  $param{db} = q{sas}; 
+  $param{object_name_max_length} = $param{object_name_max_length} || 32;
 
   $self = $class->SUPER::new(%param);
   bless( $self, $class );
@@ -47,21 +47,6 @@ sub new {
 }
 
 
-=head2 _get_drop_index_sql
-
-Create drop index sql for given index. Discard tablename.
-
-(same as postgres)
-
-=cut
-
-sub _get_drop_index_sql {
-  my ( $self, $tablename, $indexname ) = @_;
-  return qq{drop index $indexname cascade}
-    . $self->{end_of_statement}
-    . $self->{newline};
-}
-
 1;
 
 __END__
